@@ -48,6 +48,9 @@ getTableNames <- function(connection, databaseSchema) {
   } else {
     if (connection@dbms == "oracle") {
       databaseSchema <- toupper(databaseSchema)
+    } 
+    if (connection@dbms == "redshift") {
+      databaseSchema <- tolower(databaseSchema)
     }
     databaseSchema <- strsplit(databaseSchema, "\\.")[[1]]
     if (length(databaseSchema) == 1) {
