@@ -196,10 +196,14 @@ getServer <- function(connection) {
 }
 
 getServer.default <- function(connection) {
-  databaseMetaData <- rJava::.jcall(connection@jConnection,
-                                    "Ljava/sql/DatabaseMetaData;",
-                                    "getMetaData")
-  url <- rJava::.jcall(databaseMetaData, "Ljava/lang/String;", "getURL")
+  if (connection@dbms == "hive") {
+    url <- connection@url
+  } else {
+    databaseMetaData <- rJava::.jcall(connection@jConnection,
+                                      "Ljava/sql/DatabaseMetaData;",
+                                      "getMetaData")
+    url <- rJava::.jcall(databaseMetaData, "Ljava/lang/String;", "getURL")
+  }
   server <- urltools::url_parse(url)$domain
   return(server)
 }
@@ -216,8 +220,13 @@ compileReconnectCode.default <- function(connection) {
   databaseMetaData <- rJava::.jcall(connection@jConnection,
                                     "Ljava/sql/DatabaseMetaData;",
                                     "getMetaData")
-  url <- rJava::.jcall(databaseMetaData, "Ljava/lang/String;", "getURL")
-  user <- rJava::.jcall(databaseMetaData, "Ljava/lang/String;", "getUserName")
+  if (connection@dbms == 'hive') {
+    url <- connection@url
+    user <- connection@user
+  } else {
+    url <- rJava::.jcall(databaseMetaData, "Ljava/lang/String;", "getURL")
+    user <- rJava::.jcall(databaseMetaData, "Ljava/lang/String;", "getUserName")
+  }
   code <- sprintf("library(DatabaseConnector)\ncon <- connect(dbms = \"%s\", connectionString = \"%s\", user = \"%s\", password = password)",
                   connection@dbms,
                   url,
