@@ -10,8 +10,7 @@ test_that("Send updates to server", {
   details <- createConnectionDetails(dbms = "postgresql",
                                      user = Sys.getenv("CDM5_POSTGRESQL_USER"),
                                      password = URLdecode(Sys.getenv("CDM5_POSTGRESQL_PASSWORD")),
-                                     server = Sys.getenv("CDM5_POSTGRESQL_SERVER"),
-                                     schema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA"))
+                                     server = Sys.getenv("CDM5_POSTGRESQL_SERVER"))
   connection <- connect(details)
   
   expect_null(renderTranslateExecuteSql(connection, sql))
@@ -19,13 +18,12 @@ test_that("Send updates to server", {
   expect_null(renderTranslateExecuteSql(connection, sql, runAsBatch = TRUE))
   
   disconnect(connection)
-
+  
   # SQL Server
   details <- createConnectionDetails(dbms = "sql server",
                                      user = Sys.getenv("CDM5_SQL_SERVER_USER"),
                                      password = URLdecode(Sys.getenv("CDM5_SQL_SERVER_PASSWORD")),
-                                     server = Sys.getenv("CDM5_SQL_SERVER_SERVER"),
-                                     schema = Sys.getenv("CDM5_SQL_SERVER_CDM_SCHEMA"))
+                                     server = Sys.getenv("CDM5_SQL_SERVER_SERVER"))
   connection <- connect(details)
   
   expect_null(renderTranslateExecuteSql(connection, sql))
@@ -33,13 +31,12 @@ test_that("Send updates to server", {
   expect_null(renderTranslateExecuteSql(connection, sql, runAsBatch = TRUE))
   
   disconnect(connection)
-
+  
   # Oracle
   details <- createConnectionDetails(dbms = "oracle",
                                      user = Sys.getenv("CDM5_ORACLE_USER"),
                                      password = URLdecode(Sys.getenv("CDM5_ORACLE_PASSWORD")),
-                                     server = Sys.getenv("CDM5_ORACLE_SERVER"),
-                                     schema = Sys.getenv("CDM5_ORACLE_CDM_SCHEMA"))
+                                     server = Sys.getenv("CDM5_ORACLE_SERVER"))
   connection <- connect(details)
   
   expect_null(renderTranslateExecuteSql(connection, sql))
@@ -47,10 +44,10 @@ test_that("Send updates to server", {
   expect_null(renderTranslateExecuteSql(connection, sql, runAsBatch = TRUE))
   
   disconnect(connection)
-
+  
   # # RedShift details <- createConnectionDetails(dbms = 'redshift', user =
   # Sys.getenv('CDM5_REDSHIFT_USER'), password = URLdecode(Sys.getenv('CDM5_REDSHIFT_PASSWORD')),
   # server = Sys.getenv('CDM5_REDSHIFT_SERVER'), schema = Sys.getenv('CDM5_REDSHIFT_CDM_SCHEMA'))
   # connection <- connect(details) expect_true(inherits(connection, 'DatabaseConnectorConnection'))
   # expect_true(disconnect(connection))
-})
+})
\ No newline at end of file
